@@ -48,8 +48,12 @@ const REFRESH_FREQ = 1000;
 const REFRESH_NFREQ = 1000 / REFRESH_FREQ;
 const STREAMS_FREQ = 200;
 
-let Prompts = {};
+let Prompts = {}, Preallocator = {};
 Components.utils.import('resource://dta/prompts.jsm', Prompts);
+Components.utils.import('resource://dta/preallocator.jsm', Preallocator);
+Components.utils.import('resource://dta/loggedprompter.jsm'); 
+
+const AuthPrompts = new LoggedPrompter(window); 
 
 var TEXT_PAUSED;
 var TEXT_QUEUED;
@@ -407,6 +411,7 @@ var Dialog = {
 			Debug.logString("Let's resume " + download + " at " + download.partialSize);
 		}
 		this._running.push(new Dialog.RunningJob(download));
+		download.prealloc();
 		download.resumeDownload();
 	},
 	wasStopped: function D_wasStopped(download) {
@@ -525,6 +530,8 @@ var Dialog = {
 				else if (d.is(FINISHING)) {
 					++finishing;
 				}
+				d.cancelPreallocation();
+				return true;
 			},
 			this
 		);
@@ -536,7 +543,7 @@ var Dialog = {
 			}
 			Debug.logString("Going down even if queue was not probably closed yet!");
 		}
-		close();
+		close(); // XXX #1267: closeWindow(true);
 		return true;
 	},
 	_cleanTmpDir: function D__cleanTmpDir() {
@@ -600,7 +607,7 @@ function UrlManager(urls) {
 UrlManager.prototype = {
 	_sort: function(a,b) {
 		const rv = b.preference - a.preference;
-		return rv ? rv : (a.url < b.url ? -1 : 1);
+		return rv ? rv : (Math.floor(Math.random() * 3) - 1);
 	},
 	initByArray: function um_initByArray(urls) {
 		for each (let u in urls) {
@@ -685,8 +692,8 @@ function Visitor() {
 	}
 
 	let nodes = arguments[0];
-	for (x in nodes) {
-		if (!name || !(name in this.cmpKeys))	{
+	for (let x in nodes) {
+		if (!(x in this.cmpKeys))	{
 			continue;
 		}
 		this[x] = nodes[x];
@@ -699,7 +706,7 @@ Visitor.prototype = {
 			return;
 		}
 
-		for (x in this.cmpKeys) {
+		for (let x in this.cmpKeys) {
 			// we don't have this header
 			if (!(x in this)) {
 				continue;
@@ -723,7 +730,7 @@ Visitor.prototype = {
 	save: function vi_save(node) {
 		var rv = {};
 		// salva su file le informazioni sugli headers
-		for (x in this.cmpKeys) {
+		for (let x in this.cmpKeys) {
 			if (!(x in this)) {
 				continue;
 			}
@@ -978,10 +985,9 @@ VisitorManager.prototype = {
 	 * @author Nils
 	 */
 	get time() {
-		for (let i in this._visitors) {
-			let v = this._visitors[i];
+		for each (let v in this._visitors) {
 			if (v.time && v.time > 0) {
-				return this._visitors[i].time;
+				return v.time;
 			}
 		}
 		throw new Exception("No Date registered");
@@ -1012,9 +1018,10 @@ QueueItem.prototype = {
 			// remove ourself from inprogresslist
 			Dialog.wasStopped(this);
 		}
+		this.speed = '';
 		this._state = nv;
-		this.invalidate();
 		Dialog.signal(this);
+		this.invalidate();
 		Tree.refreshTools();
 		return nv;
 	},
@@ -1201,13 +1208,11 @@ QueueItem.prototype = {
 	_totalSize: 0,
 	get totalSize() { return this._totalSize; },
 	set totalSize(nv) {
-		if (this._totalSize == nv) {
-			return nv;
-		}
 		if (nv >= 0 && !isNaN(nv)) {
 			this._totalSize = Math.floor(nv);
 		}
 		this.invalidate();
+		this.prealloc();
 		return this._totalSize;
 	},
 	partialSize: 0,
@@ -1271,13 +1276,19 @@ QueueItem.prototype = {
 	},
 	get size() {
 		try {
-			let file = new FileFactory(this.destinationFile);
-			if (file.exists()) {
+			let file = null;
+			if (!this.isOf(COMPLETE, FINISHING)) {
+				file = this._tmpFile || null;	
+			}
+			else {
+				file = new FileFactory(this.destinationFile);
+			}
+			if (file && file.exists()) {
 				return file.fileSize;
 			}
 		}
 		catch (ex) {
-			Debug.log("download::getSize(): ", e)
+			Debug.log("download::getSize(): ", ex);
 		}
 		return 0;
 	},
@@ -1640,8 +1651,9 @@ QueueItem.prototype = {
 				message = _("canceled");
 			}
 			this.status = message;
-
-
+			
+			this.cancelPreallocation();
+			
 			this.removeTmpFile();
 
 			// gc
@@ -1659,6 +1671,55 @@ QueueItem.prototype = {
 		}
 	},
 	
+	prealloc: function QI_prealloc() {
+		let file = this.tmpFile;
+		
+		if (!this.is(RUNNING)) {
+			return false;
+		}
+		
+		if (!this.totalSize) {
+			Debug.logString("pa: no totalsize");
+			return false;
+		}
+		if (this.preallocating) {
+			Debug.logString("pa: already working");
+			return true;
+		}
+		
+		if (!file.exists() || this.totalSize != this.size) {
+			if (!file.parent.exists()) {
+				file.parent.create(Ci.nsIFile.DIRECTORY_TYPE, Prefs.dirPermissions);
+			}
+			let pa = Preallocator.prealloc(file, this.totalSize, Prefs.permissions, this._donePrealloc, this);
+			if (pa) {
+				this.preallocating = true;
+				this._preallocator = pa;
+				Debug.logString("started");
+			}
+		}
+		else {
+			Debug.logString("pa: already allocated");
+		}
+		return this.preallocating;
+	},
+	cancelPreallocation: function() {
+		if (this._preallocator) {
+			this._preallocator.cancel();
+		}
+		this.preallocating = false;
+	},
+	
+	_donePrealloc: function QI__donePrealloc(res) {
+		Debug.logString("pa: done");
+		delete this._preallocator; 
+		this.preallocating = false;
+		if (this.resumable && this.is(RUNNING)) {
+			this.resumeDownload();
+		}
+	},
+	
+	
 	removeTmpFile: function QI_removeTmpFile() {
 		if (!!this._tmpFile && this._tmpFile.exists()) {
 			try {
@@ -1750,6 +1811,10 @@ QueueItem.prototype = {
 			let paused = this.chunks.filter(function (chunk) !(chunk.running || chunk.complete));
 			
 			while (this.activeChunks < this.maxChunks) {
+				if (this.preallocating && this.activeChunks) {
+					Debug.logString("not resuming download " + this + " because preallocating");
+					return true;
+				}
 
 				// restart paused chunks
 				if (paused.length) {
@@ -1932,32 +1997,9 @@ Chunk.prototype = {
 		let file = this.parent.tmpFile;
 		if (!file.parent.exists()) {
 			file.parent.create(Ci.nsIFile.DIRECTORY_TYPE, Prefs.dirPermissions);
-			this.parent.invalidate();
-		}
-		let prealloc = !file.exists();
-		if (prealloc && this.parent.totalSize > 0) {
-			try {
-				file.create(file.NORMAL_FILE_TYPE, Prefs.permissions);
-				file.fileSize = this.parent.totalSize;
-				Debug.logString("fileSize set using #1");
-				prealloc = false;
-			}
-			catch (ex) {
-				// no op
-			}
 		}		
 		let outStream = new FileOutputStream(file, 0x02 | 0x08, Prefs.permissions, 0);
 		let seekable = outStream.QueryInterface(Ci.nsISeekableStream);
-		if (prealloc && this.parent.totalSize > 0) {
-			try {
-				seekable.seek(0x00, this.parent.totalSize);
-				seekable.setEOF();
-				Debug.logString("fileSize set using #2");
-			}
-			catch (ex) {
-				// no-op
-			}
-		}
 		seekable.seek(0x00, this.start + this.written);
 		this._outStream = new BufferedOutputStream(outStream, CHUNK_BUFFER_SIZE);
 	},
@@ -1994,7 +2036,7 @@ Chunk.prototype = {
 			if (!this._outStream) {
 				this.open();
 			}
-			bytes = this.remainder;
+			let bytes = this.remainder;
 			if (!this.total || aCount < bytes) {
 				bytes = aCount;
 			}
@@ -2038,25 +2080,6 @@ Chunk.prototype = {
 	}
 }
 
-let AuthPrompts = {
-	_authPrompter: null,
-	_prompter: null,
-	get authPrompter() {
-		if (!this._authPrompter) {
-			this._authPrompter = WindowWatcherService.getNewAuthPrompter(window)
-				.QueryInterface(Ci.nsIAuthPrompt);		
-		}
-		return this._authPrompter;
-	},
-	get prompter() {
-		if (!this._prompter) {
-			this._prompter = WindowWatcherService.getNewPrompter(window)
-				.QueryInterface(Ci.nsIPrompt);
-		}
-		return this._prompter;
-	}
-};
-
 function Connection(d, c, getInfo) {
 
 	this.d = d;
@@ -2206,11 +2229,34 @@ Connection.prototype = {
 
 	// nsIChannelEventSink
 	onChannelRedirect: function DL_onChannelRedirect(oldChannel, newChannel, flags) {
+		let c = this.c;
+		try {
+			if (!(oldChannel instanceof Ci.nsIChannel) || !(newChannel instanceof Ci.nsIChannel)) {
+				throw new Exception("redirect: requests not channels");
+			}
+			
+			// When we get redirected from, say, http to ftp, we need to explicitly
+			// call resumeAt() as this won't be propagated from the old channel.
+			if (c.start + c.written > 0 && !(newChannel instanceof Ci.nsIHttpChannel)) {
+				let resumable = newChannel.QueryInterface(Ci.nsIResumableChannel);
+				resumable.resumeAt(c.start + c.written, '');
+				Debug.logString("redirect: set resumeAt on " + newChannel.URI.spec + "/" + newChannel.originalURI.spec + " at " + (c.start + c.written));
+			}
+		}
+		catch (ex) {
+			Debug.log("redirect: cannot resumeAt", ex);
+			if (!this.handleError()) {
+				d.fail(_('servererror'), _('ftperrortext'), _('servererror'));
+				return;
+			}
+		}
+			
+		this._chan = newChannel;
+		
 		if (!this.isInfoGetter) {
 			return;
 		}
 		try {
-			this._chan == newChannel;
 			let newurl = new DTA_URL(newChannel.URI.QueryInterface(Ci.nsIURL), this.url.preference);
 			this.d.urlManager.replace(this.url, newurl);
 			this.url = newurl;
@@ -2453,9 +2499,21 @@ Connection.prototype = {
 		let d = this.d;
 		try {
 			let pb = aChannel.QueryInterface(Ci.nsIPropertyBag2);
-			d.totalSize = Math.max(pb.getPropertyAsInt64('content-length'), 0);
+			let totalSize = Math.max(pb.getPropertyAsInt64('content-length'), 0);
+			if (d.totalSize && totalSize != this.totalSize && !this.handleError()) {
+				Debug.logString("ftp: total size mismatch " + totalSize + " " + this.totalSize);
+				d.fail(_('servererror'), _('ftperrortext'), _('servererror')); 
+				return false;
+			}
+			Debug.logString("ftp: total size is: " + totalSize + " for: " + this.url);
+			d.totalSize = totalSize;
 		}
 		catch (ex) {
+			Debug.log("ftp: no totalsize", ex);
+			if (c.start != 0 && !this.handleError()) {
+				d.fail(_('servererror'), _('ftperrortext'), _('servererror')); 
+				return false;
+			}
 			d.totalSize = 0;
 			d.resumable = false;
 		}
@@ -2547,13 +2605,12 @@ Connection.prototype = {
 			}
 
 			if (this.isInfoGetter) {
-				// Checks for available disk space.
-				
 				if (d.fileName.getExtension() == 'metalink') {
 					d.isMetalink = true;
 					d.resumable = false;
 				}				
 				
+				// Checks for available disk space.
 				var tsd = d.totalSize;
 				try {
 					if (tsd) {
@@ -2666,7 +2723,9 @@ Connection.prototype = {
 		// and tries to advance into said directory
 		if (aStatusCode == NS_ERROR_FTP_CWD) {
 			Debug.logString("Cannot change to directory :p", aStatusCode);
-			d.cancel();
+			if (!this.handleError()) {
+				d.fail(_('servererror'), _('ftperrortext'), _('servererror'));
+			}
 			return;
 		}
 			
@@ -3000,18 +3059,26 @@ var Serializer = {
 addEventListener(
 	"load",
 	function() {
-		if (Preferences.getExt('startminimized') && window.arguments && window.arguments.length != 0) {
-			setTimeout(
-				function() {
-					try {
-						window.QueryInterface(Ci.nsIDOMChromeWindow).minimize();
-					}
-					catch (ex) {
-					}
-				},
-				0
-			);
+		if (!Preferences.getExt('startminimized', false)) {
+			return;
 		}
+		// Only start minimized if invoked with new downloads
+		if (!window.arguments || !window.arguments.length) {
+			return;
+		}
+		setTimeout(
+			function() {
+				try {
+					window.QueryInterface(Ci.nsIDOMChromeWindow).minimize();
+					if (window.opener) {
+						window.opener.focus();
+					}
+				}
+				catch (ex) {
+				}
+			},
+			0
+		);
 	},
 	false
 );
\ No newline at end of file
