@@ -324,7 +324,7 @@ var Dialog = {
 
 	// dialog destruction
 	unload: function DTA_unload() {
-		self.close();
+		close(); // XXX #1267: closeWindow(true);
 		return true;
 	},
 
@@ -415,7 +415,7 @@ var Dialog = {
 		}
 
 		// if we get here some error occured - just close.
-		self.close();
+		close(); // XXX #1267: closeWindow(true);
 		return false;
 	},
 
